from sklearn.exceptions import NotFittedError


def check_is_fitted(estimator):
    """Perform is_fitted validation for estimator.
    Checks if the estimator is fitted by verifying the presence of
    fitted attributes (ending with a trailing underscore) and otherwise
    raises a NotFittedError with the message: "This %(name)s instance is not fitted
    yet. Call 'fit' with appropriate arguments before using this
    estimator.".
    This utility is meant to be used internally by estimators themselves,
    typically in their own predict methods.

    Parameters
    ----------
    estimator : estimator instance.
        estimator instance for which the check is performed.

    Returns
    -------
    None
    Raises
    ------
    NotFittedError
        If the attributes are not found.
    """
    # nochmal schauen wozu
    #if isclass(estimator):
    #    raise TypeError("{} is a class, not an instance.".format(estimator))

    msg = ("This %(name)s instance is not fitted yet. Call 'fit' with "
           "appropriate arguments before using this estimator.")

    attrs = [v for v in vars(estimator) if v.endswith("_") and not v.startswith("__")]

    if not attrs:
        raise NotFittedError(msg % {'name': type(estimator).__name__})


import pandas as pd
from os.path import dirname, join


def load_data(data_file_name):
    module_path = dirname(__file__)
    path = join(module_path, 'data', data_file_name)
    data = pd.read_csv(path,index_col="DEMAND_DATE")
    return data



import pulp
from ..utils.validation import check_is_fitted


class BigDataNewsvendor:
    def __init__(self, cp, ch, lp_type):
        self.cp = 2
        self.ch = 1
        self.lp_type = 'minimize'

    def fit(self, x, y):
        # Add intercept
        x.insert(0, 'intercept', 1)

        if self.lp_type == 'minimize':
            nvAlgo = pulp.LpProblem(sense=pulp.LpMinimize)

        elif self.lp_type == 'maximize':
            nvAlgo = pulp.LpProblem(sense=pulp.LpMaximize)

        else:
            raise NameError('lp_type must be eigther minimize or maximize')

        n = x.index.values
        p = x.columns.values

        q = pulp.LpVariable.dicts('q', p)
        u = pulp.LpVariable.dicts('u', n, lowBound=0)
        o = pulp.LpVariable.dicts('o', n, lowBound=0)

        nvAlgo += (sum([self.cp * u[i] for i in n]) + sum([self.ch * o[i] for i in n])) / len(n)

        for i in n:
            nvAlgo += u[i] >= y.loc[i] - q['intercept'] - sum([q[j] * x.loc[i, j] for j in p if j != 'intercept'])
            nvAlgo += o[i] >= q['intercept'] + sum([q[j] * x.loc[i, j] for j in p if j != 'intercept']) - y[i]

        nvAlgo.solve()

        self.q_ = q

        return self

    def predict(self, X):
        check_is_fitted(self)
        X.insert(0, 'intercept', 1)
        pred = []
        for index, row in X.iterrows():
            # access data using column names
            value = 0
            for name in X.columns:
                value = value + row[name] * self.q_[name].value()
            pred += [value]
        return pred


from setuptools import setup, find_packages

with open('README.md') as f:
    README = f.read()

setup(
    name='ddop',
    version='v0.0.2',
    url='',
    license='MIT',
    author='Andreas Philippi',
    author_email='',
    description='Package for data-driven operations management',
    long_description=README,
    include_package_data=True,
    packages=find_packages(),
    python_requires=">=3.7",
    install_requires=['sklearn>=0.0','pandas','PuLP==2.0', 'tensorflow==2.1.0', 'Keras==2.3.1']
)




def cost(cp, ch, Y_pred, Y_true):
    if Y_pred > Y_true:
        cost = (Y_pred - Y_true) * ch
    else:
        cost = (Y_true - Y_pred) * cp
    return cost


def costs(cp, ch, Y_pred, Y_true):
    costs = []
    for i in Y_pred:
        costs[i] = cost(cp, ch, Y_pred[i], Y_true[i])
    return costs


def avg_costs(cp, ch, Y_pred, Y_true):
    avg_costs = 0
    for i in Y_pred:
        avg_costs += cost(cp, ch, Y_pred[i], Y_true[i])
    return avg_costs


from keras.models import Sequential
from keras.layers import Dense
import keras.backend as K
from ..utils.validation import check_is_fitted


class DeepLNewsvendor:
    def __init__(self, cp, ch):
        self.cp = cp
        self.ch = ch

    def __nv_loss(self, cp, ch):
        def customized_loss(y_true, y_pred):
            loss = K.switch(K.less(y_pred, y_true), cp * (y_true - y_pred), ch * (y_pred - y_true))
            return K.sum(loss)

        return customized_loss

    def __baseline_model(self, X):
        nFeatures = X.shape[1]
        model = Sequential()
        model.add(Dense(nFeatures, activation='relu', input_dim=nFeatures))
        model.add(Dense(3 * nFeatures))
        model.add(Dense(2 * nFeatures))
        model.add(Dense(1))
        model.compile(loss=self.__nv_loss(self.cp, self.ch), optimizer='adam')
        return model

    def fit(self, X, Y):
        model = self.__baseline_model(X)
        model.fit(X, Y, epochs=500, verbose=0)
        self.model_ = model
        return self

    def predict(self, X):
        check_is_fitted(self)
        pred = self.model_.predict(X)
        pred_flatted = pred.ravel()
        return pred_flatted
